#include <mpc5748g_401.h>
#include <sprz4.h>

#define LED1	98
#define LED2	99
#define LED3	100
#define LED4	101

/*	start cpu 1 and cpu 2
 */
void start_cpu(void (*pcpu1)(void), void (*pcpu2)(void))
	{
	if (pcpu1)
		{
		MC_ME_CCTL2 = 0xFC;			/* DRUN mode */
		MC_ME_CADDR2 = (unsigned long)pcpu1;	/* reset address */
		MC_ME_CADDR2 |= 1;			/* reset request */
		}
	if (pcpu2)
		{
		MC_ME_CCTL3 = 0xFC;			/* DRUN mode */
		MC_ME_CADDR3 = (unsigned long)pcpu2;	/* reset address */
		MC_ME_CADDR3 |= 1;			/* reset request */
		}
	MC_ME_MCTL = 0x30005AF0;			/* mode transition to DRUN */
	MC_ME_MCTL = 0x3000A50F;
	while (MC_ME.GS.B.S_MTRANS)			/* wait for transition complete */
		;
	}

/*	Hardware initialization
 */
void init(void)
	{
	int i;
	extern @ext const unsigned long _svectab[];

	MC_ME.ME.R = 0x000005FF;	/* enable all modes */
	MC_ME.RUN_PC[0].R = 0x000000FE;	/* enable all periphs */
	
	system160mhz();			/* set pll */
	
	for (i = LED1; i <= LED4; ++i)
		{
		SIUL2.MSCR[i].B.SSS = 0;	/* Pin functionality as GPIO */
		SIUL2.MSCR[i].B.OBE = 1;	/* Output Buffer Enable on */      
		SIUL2.MSCR[i].B.IBE = 0;	/* Input Buffer Enable off */
		SIUL2.GPDO[i].R = 1;		/* Turn LED off */
		}

	IVPR = (unsigned long)_svectab;	/* Vector base */
	INTC_BCR = 0x0111;		// Hardware interrupts
	INTC_CPR0 = 0;			// cpu 0, prio 0
	INTC_PSR227 = 0x8001;		// cpu 0, prio 1

	PIT_MCR = 4;			/* setup timer */
	PIT_LDVAL1 = 50000;
	PIT_TFLG1 = 1;			/* clear pending interrupt */
	PIT_TCTRL1 = 3;			/* enable timer with interrupts */

	_asm(" wrteei 1");		/* enable interrupts */
	}

/*	Timer interrupt
 */
@interrupt void it_pit1(void)
	{
	extern int delayLED0, delayLED1, delayLED2;
	
	if (delayLED0)
		--delayLED0;
	if (delayLED1)
		--delayLED1;
	if (delayLED2)
		--delayLED2;
	PIT_TFLG1 = 1;			/* clear interrupt */
	INTC_EOIR0 = 0;			/* end of interrupt */
	}

/*	Toggle LED
 */
void ToggleLED(unsigned int position)
	{
	if (position & 1)  
		SIUL2.GPDO[LED1].R ^= 1;
	if (position & 2)  
		SIUL2.GPDO[LED2].R ^= 1;
	if (position & 4)  
		SIUL2.GPDO[LED3].R ^= 1;
	}
